@@ -53,14 +53,6 @@
 #include <liboil/liboilcpu.h>
 #include <liboil/liboilfunction.h>
 
-/* elementfactory information */
-static GstElementDetails gst_a52dec_details = {
-  "ATSC A/52 audio decoder",
-  "Codec/Decoder/Audio",
-  "Decodes ATSC A/52 encoded audio streams",
-  "David I. Lehn <dlehn@users.sourceforge.net>"
-};
-
 #ifdef LIBA52_DOUBLE
 #define SAMPLE_WIDTH 64
 #else
@@ -140,7 +132,10 @@ gst_a52dec_base_init (gpointer g_class)
       gst_static_pad_template_get (&sink_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_factory));
-  gst_element_class_set_details (element_class, &gst_a52dec_details);
+  gst_element_class_set_details_simple (element_class,
+      "ATSC A/52 audio decoder", "Codec/Decoder/Audio",
+      "Decodes ATSC A/52 encoded audio streams",
+      "David I. Lehn <dlehn@users.sourceforge.net>");
 
   GST_DEBUG_CATEGORY_INIT (a52dec_debug, "a52dec", 0,
       "AC3/A52 software decoder");
@@ -490,7 +485,7 @@ gst_a52dec_sink_event (GstPad * pad, GstEvent * event)
         GST_DEBUG_OBJECT (a52dec,
             "Pushing newseg rate %g, applied rate %g, format %d, start %"
             G_GINT64_FORMAT ", stop %" G_GINT64_FORMAT ", pos %"
-            G_GINT64_FORMAT, rate, arate, fmt, start, end, time);
+            G_GINT64_FORMAT, rate, arate, fmt, start, end, pos);
 
         ret = gst_pad_push_event (a52dec->srcpad, event);
       }
