@@ -65,7 +65,7 @@ typedef struct _GstXingSeekEntry
 } GstXingSeekEntry;
 
 static inline GstXingSeekEntry *
-gst_xing_seek_entry_new ()
+gst_xing_seek_entry_new (void)
 {
   return g_slice_new (GstXingSeekEntry);
 }
@@ -239,7 +239,7 @@ has_xing_header (guint32 header, guchar * data, gsize size)
 static GstBuffer *
 generate_xing_header (GstXingMux * xing)
 {
-  guint32 *xing_flags;
+  guint8 *xing_flags;
   guint32 xing_flags_tmp = 0;
   GstBuffer *xing_header;
   guchar *data;
@@ -288,7 +288,7 @@ generate_xing_header (GstXingMux * xing)
   memcpy (data, "Xing", 4);
   data += 4;
 
-  xing_flags = (guint32 *) data;
+  xing_flags = data;
   data += 4;
 
   if (xing->duration != GST_CLOCK_TIME_NONE) {
@@ -379,7 +379,7 @@ generate_xing_header (GstXingMux * xing)
     }
   }
 
-  GST_DEBUG ("Setting Xing flags to 0x%x\n", *xing_flags);
+  GST_DEBUG ("Setting Xing flags to 0x%x\n", xing_flags_tmp);
   xing_flags_tmp = GUINT32_TO_BE (xing_flags_tmp);
   memcpy (xing_flags, &xing_flags_tmp, 4);
   return xing_header;
@@ -390,12 +390,6 @@ gst_xing_mux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  static const GstElementDetails gst_xing_mux_details =
-      GST_ELEMENT_DETAILS ("MP3 Xing muxer",
-      "Formatter/Metadata",
-      "Adds a Xing header to the beginning of a VBR MP3 file",
-      "Christophe Fergeau <teuf@gnome.org>");
-
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_xing_mux_src_template));
   gst_element_class_add_pad_template (element_class,
@@ -403,7 +397,10 @@ gst_xing_mux_base_init (gpointer g_class)
 
   GST_DEBUG_CATEGORY_INIT (xing_mux_debug, "xingmux", 0, "Xing Header Muxer");
 
-  gst_element_class_set_details (element_class, &gst_xing_mux_details);
+  gst_element_class_set_details_simple (element_class, "MP3 Xing muxer",
+      "Formatter/Metadata",
+      "Adds a Xing header to the beginning of a VBR MP3 file",
+      "Christophe Fergeau <teuf@gnome.org>");
 }
 
 static void
