@@ -44,13 +44,6 @@
 GST_DEBUG_CATEGORY_STATIC (rtspreal_debug);
 #define GST_CAT_DEFAULT (rtspreal_debug)
 
-/* elementfactory information */
-static const GstElementDetails rtspreal_details =
-GST_ELEMENT_DETAILS ("RealMedia RTSP Extension",
-    "Network/Extension/Protocol",
-    "Extends RTSP so that it can handle RealMedia setup",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 #define SERVER_PREFIX "RealServer"
 #define DEFAULT_BANDWIDTH	"10485800"
 
@@ -212,11 +205,16 @@ G_STMT_START {			       \
   }                                    \
 } G_STMT_END
 
+/* FIXME: remove once we depend on GLib >= 2.20 */
+#if GLIB_CHECK_VERSION (2, 20, 0)
+#define gst_rtsp_base64_decode_ip g_base64_decode_inplace
+#endif
+
 #define READ_BUFFER_GEN(src, func, name, dest, dest_len)    \
 G_STMT_START {			                            \
-  dest = (gchar *)func (src, name);                                  \
+  dest = (gchar *)func (src, name);                         \
   if (!dest) {                                              \
-    dest = "";                                              \
+    dest = (char *) "";                                     \
     dest_len = 0;                                           \
   }                                                         \
   else if (!strncmp (dest, "buffer;\"", 8)) {               \
@@ -250,11 +248,11 @@ G_STMT_START {			                          \
 G_STMT_START {			                              \
   const gchar *val = gst_sdp_media_get_attribute_val (media, name); \
   if (val && !strncmp (val, "string;\"", 8)) {                \
-    dest = (gchar *) val + 8;                                           \
+    dest = (gchar *) val + 8;                                 \
     dest_len = strlen (dest) - 1;                             \
     dest[dest_len] = '\0';                                    \
   } else {                                                    \
-    dest = "";                                                \
+    dest = (char *) "";                                       \
     dest_len = 0;                                             \
   }                                                           \
 } G_STMT_END
@@ -324,6 +322,7 @@ rtsp_ext_real_parse_sdp (GstRTSPExtension * ext, GstSDPMessage * sdp,
     ctx->duration = MAX (ctx->duration, intval);
   }
 
+  /* FIXME: use GstByteWriter to write the header */
   /* PROP */
   offset = 0;
   size = 50;
@@ -352,7 +351,7 @@ rtsp_ext_real_parse_sdp (GstRTSPExtension * ext, GstSDPMessage * sdp,
   READ_BUFFER (sdp, "Comment", comment, comment_len);
   READ_BUFFER (sdp, "Copyright", copyright, copyright_len);
 
-  size = 22 + title_len + author_len + comment_len + copyright_len;
+  size = 18 + title_len + author_len + comment_len + copyright_len;
   ENSURE_SIZE (offset + size);
   datap = data + offset;
 
@@ -369,7 +368,8 @@ rtsp_ext_real_parse_sdp (GstRTSPExtension * ext, GstSDPMessage * sdp,
   /* fix the hashtale for the rule parser */
   rules = g_string_new ("");
   vars = g_hash_table_new (g_str_hash, g_str_equal);
-  g_hash_table_insert (vars, "Bandwidth", DEFAULT_BANDWIDTH);
+  g_hash_table_insert (vars, (gchar *) "Bandwidth",
+      (gchar *) DEFAULT_BANDWIDTH);
 
   /* MDPR */
   for (i = 0; i < ctx->n_streams; i++) {
@@ -593,7 +593,9 @@ rtsp_ext_real_parse_sdp (GstRTSPExtension * ext, GstSDPMessage * sdp,
   GST_BUFFER_SIZE (buf) = offset;
 
   /* Set on caps */
+  GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_IN_CAPS);
   gst_structure_set (props, "config", GST_TYPE_BUFFER, buf, NULL);
+  gst_buffer_unref (buf);
 
   /* Overwrite encoding and media fields */
   gst_structure_set (props, "encoding-name", G_TYPE_STRING, "X-REAL-RDT", NULL);
@@ -667,6 +669,7 @@ reset:
 
 static void gst_rtsp_real_extension_init (gpointer g_iface,
     gpointer iface_data);
+static void gst_rtsp_real_finalize (GObject * obj);
 
 static void
 _do_init (GType rtspreal_type)
@@ -689,12 +692,19 @@ gst_rtsp_real_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_set_details (element_class, &rtspreal_details);
+  gst_element_class_set_details_simple (element_class,
+      "RealMedia RTSP Extension", "Network/Extension/Protocol",
+      "Extends RTSP so that it can handle RealMedia setup",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
 gst_rtsp_real_class_init (GstRTSPRealClass * g_class)
 {
+  GObjectClass *gobject_class = (GObjectClass *) g_class;
+
+  gobject_class->finalize = gst_rtsp_real_finalize;
+
   GST_DEBUG_CATEGORY_INIT (rtspreal_debug, "rtspreal", 0,
       "RealMedia RTSP extension");
 }
@@ -705,6 +715,29 @@ gst_rtsp_real_init (GstRTSPReal * rtspreal, GstRTSPRealClass * klass)
   rtspreal->isreal = FALSE;
 }
 
+static void
+gst_rtsp_stream_free (GstRTSPRealStream * stream)
+{
+  g_free (stream->stream_name);
+  g_free (stream->mime_type);
+  gst_asm_rule_book_free (stream->rulebook);
+  g_free (stream->type_specific_data);
+
+  g_free (stream);
+}
+
+static void
+gst_rtsp_real_finalize (GObject * obj)
+{
+  GstRTSPReal *r = (GstRTSPReal *) obj;
+
+  g_list_foreach (r->streams, (GFunc) gst_rtsp_stream_free, NULL);
+  g_list_free (r->streams);
+  g_free (r->rules);
+
+  G_OBJECT_CLASS (parent_class)->finalize (obj);
+}
+
 static void
 gst_rtsp_real_extension_init (gpointer g_iface, gpointer iface_data)
 {
